DF_AUTH_APPS = [
    "df_auth",
    "rest_framework_simplejwt",
    "django_otp",
    "django_otp.plugins.otp_email",
    "django_otp.plugins.otp_totp",
    "django_otp.plugins.otp_static",
    "otp_twilio",
    "social_django",
]
